#!/usr/bin/env python3

from keras.models import Sequential
from keras.layers import Dense
import numpy
import numpy.random as rand
# from matplotlib import pyplot
from keras.models import model_from_json
import math
import sys
import json
import tornado.ioloop
import tornado.web

nn = []
alpha = 2.5 
betha = 1.5

MIN = 0.0
MAX = 4000.0

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        try: 
            param = self.get_query_argument("smartdata")
            js = self.process_request( param )
        except:
            js = "{\"error\":\"Parameter smartdata (json) missing\"}"
        self.set_header('Content-Type', 'application/json')
        self.write( js )

    def post(self):
        try:
            param = self.get_body_argument("smartdata")
            js = self.process_request( param )
        except:
            js = "{\"error\":\"Parameter smartdata (json) missing\"}"
        self.set_header('Content-Type', 'application/json')
        self.write( js )

    def process_request(self, x): 
        rn = None
        if ( x ):
            js = json.loads( x.strip() )
            # First step: find the network and make the prediction, correcting input if 
            for net in nn:
                n = net[0]
                if n["x"] == js["x"] and n["y"] == js["y"] and n["z"] == js["z"] and n["unit"] == js["unit"] :
                    pred, conf = self.predict(js, net)
                    rn = "{\"confidence\":"+str(conf)+"}"
                    if conf < 50 and net[5]:
                        js["value"] = pred
                    net[5] = True
            for net in nn:  # update cache entry for every ANN that uses this value as input
                self.update_cache(js, net)
            if rn is None :
                rn = "{\"error\":\"No predicter found for this smartdata\"}"
        else :
            rn = "{\"Error\":\"No json received\"}"
        return rn

    def predict(self, js, net):
        input = numpy.array(net[4])
        input = numpy.reshape(input, (1, len(input)))
        # a more general normalization routine is necessary. For this example, using fixed values!
        input = input / MAX
        vi, cv = self.run_predictor( net[1], input, js["value"], net[3] )
        return vi, cv

    def run_predictor(self, model, inputs, expected_value, MAE):
        expected_value = expected_value / MAX
        aux = model.predict( inputs )
        pv = aux[0,0]
        delta = math.fabs(expected_value - pv)        
        if delta <= (betha * MAE):
            c = 1
        else :
            c = 1 - (( math.fabs(delta - (betha * MAE)) ) / ( alpha  * MAE ))
        if c < 0:
            c = 0
        pv = pv * MAX
        c = c * 100
        return pv, c

    def update_cache(self, js, net):
        pos = 0
        for input in net[2]:
            if js["x"] == input["x"] and js["y"] == input["y"] and js["z"] == input["z"] and js["unit"] == input["unit"] :
                net[4][pos] = js["value"]
            pos = pos + 1
   
def make_app():
    return tornado.web.Application([
        (r"/confidence", MainHandler),
    ])

def make_predictor(sensor_addr, netfile, weightfile, mapfile, mae):
    model = []
    model.append( json.loads( sensor_addr ) )	# model[0] is json for sensor to be predicted
    jf = open(netfile, "r")
    desc = jf.read()
    m = model_from_json(desc)
    m.load_weights(weightfile)
    m.compile(loss='mean_absolute_error', optimizer='adam', metrics=['accuracy'])
    model.append(m)				# model[1] is the neural network

    instream = open(mapfile)
    map = []
    for x in instream.readlines():
        x = x.strip()
        if len(x) > 0:
            if not x[0] == '#':  
                map.append( json.loads( x ) )

    model.append( map )                         # model[2] is the input map, determining the input order for the neural network (same order of cached values)

    model.append( float(mae) )                  # model[3] is the MAE (Mean Absolute Error), obtained in the training/test process, used to determine the confidence of predictions.

    model.append([])                            # model[4] is the values cache
    for i in range( len(map) ):
        model[4].append(0.0)

    model.append( False )                       # model[5] flag if has value in cache... If not, cannot validate predicted x read value

    return model

def load_predictors():
    nn = []
    conf = open("config.txt")    
    for x in conf.readlines():
        x = x.strip()
        if len(x) > 0 :
            if not ( x[0] == '#')  :
                netcfg = x.split(';')	
                nn.append ( make_predictor( netcfg[0], netcfg[1], netcfg[2], netcfg[3], netcfg[4] ) )
    return ( nn )

if __name__ == "__main__":
    nn = load_predictors()
    app = make_app()
    app.listen(8989)
    print("Server ready and accepting connection on port 8989...")
    tornado.ioloop.IOLoop.current().start()

