#!/usr/bin/python3
import time, requests, json, os, ssl, argparse
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager

parser = argparse.ArgumentParser(description='Get data to fulfill input buffer')

required = parser.add_argument_group('required named arguments')
required.add_argument('-s','--smartdata', help='The original smartdata', default=500)

# workflow configuration parameters
workflow_expiry = 10*1000000 # 10 sec
workflow_id     = 4
radius          = 0
domain          = 'aqtech'
username        = 'aqtech'
password        = 'jP#55Mh@'

names = ['Nacelle position',
    'RotorSpeed',
    'WindSpeed',
    'Temp. Nacelle',
    'Temp. bearing A',
    'Temp. bearing B',
    'Temp. control box axis 1',
    'Temp. control box axis 2',
    'Temp. control box axis 3',
    'Temp. gear box bearing',
    'Temp. gearbox',
    'Temp. generator 1',
    'Temp. generator 2',
    'Temp. generator cooling air',
    'Temp. hub',
    'Temp. main box',
    'Temp. shaft bearing',
    'Temp. top box']

units = {
    'Nacelle position'                  : 2224441636, 
    'Power'                             : 2224180004, 
    'RotorSpeed'                        : 2147485184, 
    'WindSpeed'                         : 2224437540, 
    'Temp. Nacelle'                     : 2224179556,
    'Temp. bearing A'                   : 2224179556,
    'Temp. bearing B'                   : 2224179556,
    'Temp. control box axis 1'          : 2224179556,
    'Temp. control box axis 2'          : 2224179556,
    'Temp. control box axis 3'          : 2224179556,
    'Temp. gear box bearing'            : 2224179556,
    'Temp. gearbox'                     : 2224179556,
    'Temp. generator 1'                 : 2224179556,
    'Temp. generator 2'                 : 2224179556,
    'Temp. generator cooling air'       : 2224179556,
    'Temp. hub'                         : 2224179556,
    'Temp. main box'                    : 2224179556,
    'Temp. shaft bearing'               : 2224179556,
    'Temp. top box'                     : 2224179556
}

devs = {
    'Nacelle position'                  : 3,
    'Power'                             : 0,
    'RotorSpeed'                        : 1,
    'WindSpeed'                         : 2,
    'Temp. Nacelle'                     : 9,
    'Temp. bearing A'                   : 6,
    'Temp. bearing B'                   : 7,
    'Temp. control box axis 1'          : 16,
    'Temp. control box axis 2'          : 17,
    'Temp. control box axis 3'          : 18,
    'Temp. gear box bearing'            : 11,
    'Temp. gearbox'                     : 8,
    'Temp. generator 1'                 : 4,
    'Temp. generator 2'                 : 5,
    'Temp. generator cooling air'       : 10,
    'Temp. hub'                         : 19,
    'Temp. main box'                    : 13,
    'Temp. shaft bearing'               : 12,
    'Temp. top box'                     : 14
}

def mount_query(timestamp, unit, dev, r, x, y, z):
    query = {
        'series': {
            'version': 1.1,
            't0': timestamp - workflow_expiry,
            't1': timestamp + workflow_expiry,
            'unit': unit, #should use the same unit as the workflow (or be passed as parameter)
            'dev': dev,
            'r': r,
            'x': x,
            'y': y,
            'z': z,
            'workflow': workflow_id #should be the same as the workflow
        },
        'credentials': {
            'domain': domain
            'username' : username
            'password' : password
        }
    }

class MyAdapter(HTTPAdapter):
    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(num_pools=connections,
                                       maxsize=maxsize,
                                       block=block,
                                       ssl_version=ssl.PROTOCOL_TLSv1)

args = vars(parser.parse_args())
smartdata = json.loads(args['smartdata'])
queries = []
for data in names:
    queries.append(mount_query(int(smartdata["timestamp"]), 
                    unit[data], 
                    dev[data], 
                    radius, 
                    smartdata["x"], 
                    smartdata["y"], 
                    smartdata["z"]))

get_url ='https://iot.ufsc.br/api/get.php'

#CLIENT_CERTIFICATE = ['client-CERT.pem', 'client-CERT.key']
session = requests.Session()
session.mount('https://', MyAdapter())
session.headers = {'Content-type' : 'application/json'}
#session.cert = CLIENT_CERTIFICATE
responses = []
count = 0
for query in queries:
    response = session.post(get_url, json.dumps(query))
    if (count == 1):
        responses.append(smartdata['value'])
        count += 1

    if response.status_code == 200:
        data = response.json()
        avg = 0
        size = len(data['series'])
        for i in range(0, size):
            avg += float(data['series'][i]['value'])
            if (size == 0):
                with open('in2_input', 'a') as fifo:
                    fifo.write("error\n")
                    fifo.flush()
                break
            else:
                responses.append(float(avg/size))
    else:
        with open('in2_input', 'a') as fifo:
            fifo.write("error\n")
            fifo.flush()
        break
    count += 1

#if(len(responses) == len(names)):
# acquired from training
#min_max = [ [0.  , -19. , 0. , 0. , 9. , 11., 10., 0. , 0. , 0. , 9. , 11., 11. , 12. , 11. , 0. , 9. , 12., 21.],
#            [359., 1792., 16., 26., 44., 57., 84., 43., 42., 48., 79., 73., 131., 130., 106., 41., 42., 47., 51.]]
#scaler.fit(min_max)
#scaler.transform(responses)
with open('in2_input', 'a') as fifo:
    fifo.write(responses+'\n')
    fifo.flush()
